﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Windows.Media.Effects;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void GeoDrawing(int size, int widthCount, int heightCount)
        {
            double x1 = 10;
            double y1 = 0;

            DateTime dt1 = DateTime.Now;
            System.Windows.Shapes.Path path = new System.Windows.Shapes.Path();
            path.Stroke = Brushes.Green;
            path.StrokeThickness = 0.1;
            GeometryGroup grop = new GeometryGroup();
            int total = widthCount * heightCount;

            for (int i = 0; i < 50000; i++)
            {
                x1 = (i % widthCount) * size;
                y1 = (i / widthCount) * size;
                RectangleGeometry rectG = new RectangleGeometry();
                Rect rect = new Rect(x1, y1, size, size);
                rectG.Rect = rect;
                grop.Children.Add(rectG);
            }
            path.Data = grop;
            border1.Child = path;

            DateTime dt2 = DateTime.Now;
            System.Diagnostics.Trace.WriteLine("GeoDrawing == " + dt2.Subtract(dt1).TotalMilliseconds.ToString());
        }

        private void DrawVisual(int size, int widthCount, int heightCount)
        {
            DrawingVisual dv = new DrawingVisual();

            DateTime dt1 = DateTime.Now;

            DrawingContext dc = dv.RenderOpen();
            Pen pen = new Pen(Brushes.Black, 1.0);
            pen.Freeze();

            Random rand = new Random();

            double x1 = 0;
            double y1 = 0;

            int total = widthCount * heightCount;
            for (int i = 0; i < total; i++)
            {
                x1 = (i % widthCount) * size;
                y1 = (i / widthCount) * size;

                dc.DrawRectangle(null, pen, new Rect(x1, y1, size, size));
            }

            dc.Close();

            RenderTargetBitmap rtb = new RenderTargetBitmap(size * widthCount, size * heightCount, 96, 96, PixelFormats.Pbgra32);
            rtb.Render(dv);
            image1.Source = rtb;

            DateTime dt2 = DateTime.Now;
            System.Diagnostics.Trace.WriteLine("DrawVisual == " + dt2.Subtract(dt1).TotalMilliseconds.ToString());
        }

        // Bitmap to BitmapSource
        // http://www.codeproject.com/KB/WPF/BitmapToBitmapSource.aspx
        [DllImport("gdi32.dll")]
        private static extern bool DeleteObject(IntPtr hObject);

        private void DrawBitmap(int size, int widthCount, int heightCount)
        {
            DateTime dt1 = DateTime.Now;

            System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(size * widthCount, size * heightCount);
            System.Drawing.Graphics g = System.Drawing.Graphics.FromImage(bitmap);

            int x1 = 0;
            int y1 = 0;

            int total = widthCount * heightCount;
            System.Drawing.Pen pen = new System.Drawing.Pen(System.Drawing.Brushes.Black);
            for (int i = 0; i < total; i++)
            {
                x1 = (i % widthCount) * size;
                y1 = (i / widthCount) * size;

                g.DrawRectangle(pen, new System.Drawing.Rectangle(x1, y1, size, size));
            }

            IntPtr hBitmap = bitmap.GetHbitmap();

            try
            {
                BitmapSource bitmapSource =
                System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(hBitmap,
                IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());

                this.image2.Source = bitmapSource;
            }
            finally
            {
                DeleteObject(hBitmap);
            }

            DateTime dt2 = DateTime.Now;
            System.Diagnostics.Trace.WriteLine("DrawVisual == " + dt2.Subtract(dt1).TotalMilliseconds.ToString());
        }

        // Draw lines excactly on physical device pixels
        // http://www.wpftutorial.net/DrawOnPhysicalDevicePixels.html
        private void DrawVisual2(int size, int widthCount, int heightCount)
        {
            DrawingVisual dv = new DrawingVisual();

            Matrix m = PresentationSource.FromVisual(this)
                .CompositionTarget.TransformToDevice;
            double dpiFactor = 1 / m.M11;

            DrawingContext dc = dv.RenderOpen();
            Pen pen = new Pen(Brushes.Black, 1 * dpiFactor);
            pen.Freeze();

            Rect rect = new Rect(0, 0, size * widthCount, size * heightCount);

            double halfPenWidth = pen.Thickness / 2;

            GuidelineSet guidelines = new GuidelineSet();
            guidelines.GuidelinesX.Add(rect.Left + halfPenWidth);
            guidelines.GuidelinesX.Add(rect.Right + halfPenWidth);
            guidelines.GuidelinesY.Add(rect.Top + halfPenWidth);
            guidelines.GuidelinesY.Add(rect.Bottom + halfPenWidth);

            dc.PushGuidelineSet(guidelines);

            Random rand = new Random();

            int total = widthCount * heightCount;
            for (int i = 0; i < total; i++)
            {
                int x1 = (i % widthCount) * size;
                int y1 = (i / widthCount) * size;

                dc.DrawRectangle(null, pen, new Rect(x1, y1, size, size));
            }

            dc.Pop();
            dc.Close();

            RenderTargetBitmap rtb = new RenderTargetBitmap(size * widthCount, size * heightCount, 96, 96, PixelFormats.Pbgra32);

            rtb.Render(dv);
            rtb.Freeze();
            image3.Source = rtb;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            int size = 10;
            int widthCount = 250;
            int heightCount = 200;

            if (false)
            {
                GeoDrawing(size, widthCount, heightCount);
            }

            DrawVisual(size, widthCount, heightCount);
            DrawBitmap(size, widthCount, heightCount);
            DrawVisual2(size, widthCount, heightCount);


            Point pt = this.image2.TransformToAncestor(this).Transform(new Point(0, 0));
            System.Diagnostics.Trace.WriteLine(pt);

            pt = this.image3.TransformToAncestor(this).Transform(new Point(0, 0));
            System.Diagnostics.Trace.WriteLine(pt);

            System.Diagnostics.Trace.WriteLine(this.TranslatePoint(new Point(0, 0), this.image2));
        }
    }
}
